@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the plugins of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include <QtGui/private/qguiapplication_p.h>
 #include <QtCore/QDebug>
@@ -66,17 +30,19 @@
 #include <errno.h>
 
 #include <xcb/xfixes.h>
-#if QT_CONFIG(xkb)
 #define explicit dont_use_cxx_explicit
 #include <xcb/xkb.h>
 #undef explicit
-#endif
-#if QT_CONFIG(xcb_xinput)
 #include <xcb/xinput.h>
+
+#if QT_CONFIG(xcb_xlib)
+#include "qt_xlib_wrapper.h"
 #endif
 
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
+
 Q_LOGGING_CATEGORY(lcQpaXInput, "qt.qpa.input")
 Q_LOGGING_CATEGORY(lcQpaXInputDevices, "qt.qpa.input.devices")
 Q_LOGGING_CATEGORY(lcQpaXInputEvents, "qt.qpa.input.events")
@@ -88,14 +54,9 @@ Q_LOGGING_CATEGORY(lcQpaKeyboard, "qt.qpa.xkeyboard")
 Q_LOGGING_CATEGORY(lcQpaClipboard, "qt.qpa.clipboard")
 Q_LOGGING_CATEGORY(lcQpaXDnd, "qt.qpa.xdnd")
 
-// this event type was added in libxcb 1.10,
-// but we support also older version
-#ifndef XCB_GE_GENERIC
-#define XCB_GE_GENERIC 35
-#endif
-
 QXcbConnection::QXcbConnection(QXcbNativeInterface *nativeInterface, bool canGrabServer, xcb_visualid_t defaultVisualId, const char *displayName)
     : QXcbBasicConnection(displayName)
+    , m_duringSystemMoveResize(false)
     , m_canGrabServer(canGrabServer)
     , m_defaultVisualId(defaultVisualId)
     , m_nativeInterface(nativeInterface)
@@ -105,19 +66,15 @@ QXcbConnection::QXcbConnection(QXcbNativeInterface *nativeInterface, bool canGra
 
     m_eventQueue = new QXcbEventQueue(this);
 
-    m_xdgCurrentDesktop = qgetenv("XDG_CURRENT_DESKTOP").toLower();
-
     if (hasXRandr())
         xrandrSelectEvents();
 
-    initializeScreens();
+    initializeScreens(false);
 
-#if QT_CONFIG(xcb_xinput)
     if (hasXInput2()) {
         xi2SetupDevices();
         xi2SelectStateEvents();
     }
-#endif
 
     m_wmSupport.reset(new QXcbWMSupport(this));
     m_keyboard = new QXcbKeyboard(this);
@@ -128,10 +85,18 @@ QXcbConnection::QXcbConnection(QXcbNativeInterface *nativeInterface, bool canGra
     m_drag = new QXcbDrag(this);
 #endif
 
-    m_startupId = qgetenv("DESKTOP_STARTUP_ID");
-    if (!m_startupId.isNull())
+    setStartupId(qgetenv("DESKTOP_STARTUP_ID"));
+    if (!startupId().isNull())
         qunsetenv("DESKTOP_STARTUP_ID");
 
+    const int focusInDelay = 100;
+    m_focusInTimer.setSingleShot(true);
+    m_focusInTimer.setInterval(focusInDelay);
+    m_focusInTimer.callOnTimeout([]() {
+        // No FocusIn events for us, proceed with FocusOut normally.
+        QWindowSystemInterface::handleWindowActivated(nullptr, Qt::ActiveWindowFocusReason);
+    });
+
     sync();
 }
 
@@ -146,10 +111,9 @@ QXcbConnection::~QXcbConnection()
     if (m_eventQueue)
         delete m_eventQueue;
 
-    QXcbIntegration *integration = QXcbIntegration::instance();
     // Delete screens in reverse order to avoid crash in case of multiple screens
     while (!m_screens.isEmpty())
-        integration->destroyScreen(m_screens.takeLast());
+        QWindowSystemInterface::handleScreenRemoved(m_screens.takeLast());
 
     while (!m_virtualDesktops.isEmpty())
         delete m_virtualDesktops.takeLast();
@@ -181,15 +145,15 @@ void QXcbConnection::removeWindowEventListener(xcb_window_t id)
 
 QXcbWindowEventListener *QXcbConnection::windowEventListenerFromId(xcb_window_t id)
 {
-    return m_mapper.value(id, 0);
+    return m_mapper.value(id, nullptr);
 }
 
 QXcbWindow *QXcbConnection::platformWindowFromId(xcb_window_t id)
 {
-    QXcbWindowEventListener *listener = m_mapper.value(id, 0);
+    QXcbWindowEventListener *listener = m_mapper.value(id, nullptr);
     if (listener)
         return listener->toWindow();
-    return 0;
+    return nullptr;
 }
 
 #define HANDLE_PLATFORM_WINDOW_EVENT(event_t, windowMember, handler) \
@@ -226,6 +190,12 @@ void QXcbConnection::printXcbEvent(const QLoggingCategory &log, const char *mess
 }
 #define CASE_PRINT_AND_RETURN(name) case name : PRINT_AND_RETURN(#name);
 
+#define XI_PRINT_AND_RETURN(name) { \
+    qCDebug(log, "%s | XInput Event(%s) | sequence: %d", message, name, sequence); \
+    return; \
+}
+#define XI_CASE_PRINT_AND_RETURN(name) case name : XI_PRINT_AND_RETURN(#name);
+
     switch (response_type) {
     CASE_PRINT_AND_RETURN( XCB_KEY_PRESS );
     CASE_PRINT_AND_RETURN( XCB_KEY_RELEASE );
@@ -260,7 +230,44 @@ void QXcbConnection::printXcbEvent(const QLoggingCategory &log, const char *mess
     CASE_PRINT_AND_RETURN( XCB_COLORMAP_NOTIFY );
     CASE_PRINT_AND_RETURN( XCB_CLIENT_MESSAGE );
     CASE_PRINT_AND_RETURN( XCB_MAPPING_NOTIFY );
-    CASE_PRINT_AND_RETURN( XCB_GE_GENERIC );
+    case XCB_GE_GENERIC: {
+        if (hasXInput2() && isXIEvent(event)) {
+            auto *xiDeviceEvent = reinterpret_cast<const xcb_input_button_press_event_t*>(event); // qt_xcb_input_device_event_t
+            switch (xiDeviceEvent->event_type) {
+            XI_CASE_PRINT_AND_RETURN( XCB_INPUT_KEY_PRESS );
+            XI_CASE_PRINT_AND_RETURN( XCB_INPUT_KEY_RELEASE );
+            XI_CASE_PRINT_AND_RETURN( XCB_INPUT_BUTTON_PRESS );
+            XI_CASE_PRINT_AND_RETURN( XCB_INPUT_BUTTON_RELEASE );
+            XI_CASE_PRINT_AND_RETURN( XCB_INPUT_MOTION );
+            XI_CASE_PRINT_AND_RETURN( XCB_INPUT_ENTER );
+            XI_CASE_PRINT_AND_RETURN( XCB_INPUT_LEAVE );
+            XI_CASE_PRINT_AND_RETURN( XCB_INPUT_FOCUS_IN );
+            XI_CASE_PRINT_AND_RETURN( XCB_INPUT_FOCUS_OUT );
+            XI_CASE_PRINT_AND_RETURN( XCB_INPUT_HIERARCHY );
+            XI_CASE_PRINT_AND_RETURN( XCB_INPUT_PROPERTY );
+            XI_CASE_PRINT_AND_RETURN( XCB_INPUT_RAW_KEY_PRESS );
+            XI_CASE_PRINT_AND_RETURN( XCB_INPUT_RAW_KEY_RELEASE );
+            XI_CASE_PRINT_AND_RETURN( XCB_INPUT_RAW_BUTTON_PRESS );
+            XI_CASE_PRINT_AND_RETURN( XCB_INPUT_RAW_BUTTON_RELEASE );
+            XI_CASE_PRINT_AND_RETURN( XCB_INPUT_RAW_MOTION );
+            XI_CASE_PRINT_AND_RETURN( XCB_INPUT_TOUCH_BEGIN );
+            XI_CASE_PRINT_AND_RETURN( XCB_INPUT_TOUCH_UPDATE );
+            XI_CASE_PRINT_AND_RETURN( XCB_INPUT_TOUCH_END );
+            XI_CASE_PRINT_AND_RETURN( XCB_INPUT_TOUCH_OWNERSHIP );
+            XI_CASE_PRINT_AND_RETURN( XCB_INPUT_RAW_TOUCH_BEGIN );
+            XI_CASE_PRINT_AND_RETURN( XCB_INPUT_RAW_TOUCH_UPDATE );
+            XI_CASE_PRINT_AND_RETURN( XCB_INPUT_RAW_TOUCH_END );
+            XI_CASE_PRINT_AND_RETURN( XCB_INPUT_BARRIER_HIT );
+            XI_CASE_PRINT_AND_RETURN( XCB_INPUT_BARRIER_LEAVE );
+            default:
+                qCDebug(log, "%s | XInput Event(other type) | sequence: %d", message, sequence);
+                return;
+            }
+        } else {
+            qCDebug(log, "%s | %s(%d) | sequence: %d", message, "XCB_GE_GENERIC", response_type, sequence);
+            return;
+        }
+    }
     }
     // XFixes
     if (isXFixesType(response_type, XCB_XFIXES_SELECTION_NOTIFY))
@@ -433,7 +440,7 @@ const char *xcb_protocol_request_codes[] =
 
 void QXcbConnection::handleXcbError(xcb_generic_error_t *error)
 {
-    long result = 0;
+    qintptr result = 0;
     QAbstractEventDispatcher* dispatcher = QAbstractEventDispatcher::instance();
     if (dispatcher && dispatcher->filterNativeEvent(m_nativeInterface->nativeEventType(), error, &result))
         return;
@@ -446,21 +453,21 @@ void QXcbConnection::printXcbError(const char *message, xcb_generic_error_t *err
     uint clamped_error_code = qMin<uint>(error->error_code, (sizeof(xcb_errors) / sizeof(xcb_errors[0])) - 1);
     uint clamped_major_code = qMin<uint>(error->major_code, (sizeof(xcb_protocol_request_codes) / sizeof(xcb_protocol_request_codes[0])) - 1);
 
-    qCWarning(lcQpaXcb, "%s: %d (%s), sequence: %d, resource id: %d, major code: %d (%s), minor code: %d",
-             message,
-             int(error->error_code), xcb_errors[clamped_error_code],
-             int(error->sequence), int(error->resource_id),
-             int(error->major_code), xcb_protocol_request_codes[clamped_major_code],
-             int(error->minor_code));
+    qCDebug(lcQpaXcb, "%s: %d (%s), sequence: %d, resource id: %d, major code: %d (%s), minor code: %d",
+            message,
+            int(error->error_code), xcb_errors[clamped_error_code],
+            int(error->sequence), int(error->resource_id),
+            int(error->major_code), xcb_protocol_request_codes[clamped_major_code],
+            int(error->minor_code));
 }
 
 static Qt::MouseButtons translateMouseButtons(int s)
 {
-    Qt::MouseButtons ret = 0;
+    Qt::MouseButtons ret;
     if (s & XCB_BUTTON_MASK_1)
         ret |= Qt::LeftButton;
     if (s & XCB_BUTTON_MASK_2)
-        ret |= Qt::MidButton;
+        ret |= Qt::MiddleButton;
     if (s & XCB_BUTTON_MASK_3)
         ret |= Qt::RightButton;
     return ret;
@@ -476,7 +483,7 @@ Qt::MouseButton QXcbConnection::translateMouseButton(xcb_button_t s)
 {
     switch (s) {
     case 1: return Qt::LeftButton;
-    case 2: return Qt::MidButton;
+    case 2: return Qt::MiddleButton;
     case 3: return Qt::RightButton;
     // Button values 4-7 were already handled as Wheel events, and won't occur here.
     case 8: return Qt::BackButton;      // Also known as Qt::ExtraButton1
@@ -507,7 +514,6 @@ Qt::MouseButton QXcbConnection::translateMouseButton(xcb_button_t s)
     }
 }
 
-#if QT_CONFIG(xkb)
 namespace {
     typedef union {
         /* All XKB events share these fields. */
@@ -523,14 +529,13 @@ namespace {
         xcb_xkb_state_notify_event_t state_notify;
     } _xkb_event;
 }
-#endif
 
 void QXcbConnection::handleXcbEvent(xcb_generic_event_t *event)
 {
     if (Q_UNLIKELY(lcQpaEvents().isDebugEnabled()))
         printXcbEvent(lcQpaEvents(), "Event", event);
 
-    long result = 0; // Used only by MS Windows
+    qintptr result = 0; // Used only by MS Windows
     if (QAbstractEventDispatcher *dispatcher = QAbstractEventDispatcher::instance()) {
         if (dispatcher->filterNativeEvent(m_nativeInterface->nativeEventType(), event, &result))
             return;
@@ -544,6 +549,7 @@ void QXcbConnection::handleXcbEvent(xcb_generic_event_t *event)
         HANDLE_PLATFORM_WINDOW_EVENT(xcb_expose_event_t, window, handleExposeEvent);
     case XCB_BUTTON_PRESS: {
         auto ev = reinterpret_cast<xcb_button_press_event_t *>(event);
+        setTime(ev->time);
         m_keyboard->updateXKBStateFromCore(ev->state);
         // the event explicitly contains the state of the three first buttons,
         // the rest we need to manage ourselves
@@ -556,6 +562,9 @@ void QXcbConnection::handleXcbEvent(xcb_generic_event_t *event)
     }
     case XCB_BUTTON_RELEASE: {
         auto ev = reinterpret_cast<xcb_button_release_event_t *>(event);
+        setTime(ev->time);
+        if (m_duringSystemMoveResize && ev->root != XCB_NONE)
+            abortSystemMoveResize(ev->root);
         m_keyboard->updateXKBStateFromCore(ev->state);
         m_buttonState = (m_buttonState & ~0x7) | translateMouseButtons(ev->state);
         setButtonState(translateMouseButton(ev->detail), false);
@@ -566,6 +575,7 @@ void QXcbConnection::handleXcbEvent(xcb_generic_event_t *event)
     }
     case XCB_MOTION_NOTIFY: {
         auto ev = reinterpret_cast<xcb_motion_notify_event_t *>(event);
+        setTime(ev->time);
         m_keyboard->updateXKBStateFromCore(ev->state);
         m_buttonState = (m_buttonState & ~0x7) | translateMouseButtons(ev->state);
         if (Q_UNLIKELY(lcQpaXInputEvents().isDebugEnabled()))
@@ -573,8 +583,14 @@ void QXcbConnection::handleXcbEvent(xcb_generic_event_t *event)
                     ev->event_x, ev->event_y, ev->detail, static_cast<unsigned int>(m_buttonState));
         HANDLE_PLATFORM_WINDOW_EVENT(xcb_motion_notify_event_t, event, handleMotionNotifyEvent);
     }
-    case XCB_CONFIGURE_NOTIFY:
+    case XCB_CONFIGURE_NOTIFY: {
+        if (isAtLeastXRandR15()) {
+            auto ev = reinterpret_cast<xcb_configure_notify_event_t *>(event);
+            if (ev->event == rootWindow())
+                initializeScreens(true);
+        }
         HANDLE_PLATFORM_WINDOW_EVENT(xcb_configure_notify_event_t, event, handleConfigureNotifyEvent);
+    }
     case XCB_MAP_NOTIFY:
         HANDLE_PLATFORM_WINDOW_EVENT(xcb_map_notify_event_t, event, handleMapNotifyEvent);
     case XCB_UNMAP_NOTIFY:
@@ -586,28 +602,33 @@ void QXcbConnection::handleXcbEvent(xcb_generic_event_t *event)
         if (clientMessage->format != 32)
             return;
 #if QT_CONFIG(draganddrop)
-        if (clientMessage->type == atom(QXcbAtom::XdndStatus))
+        if (clientMessage->type == atom(QXcbAtom::AtomXdndStatus))
             drag()->handleStatus(clientMessage);
-        else if (clientMessage->type == atom(QXcbAtom::XdndFinished))
+        else if (clientMessage->type == atom(QXcbAtom::AtomXdndFinished))
             drag()->handleFinished(clientMessage);
 #endif
-        if (m_systemTrayTracker && clientMessage->type == atom(QXcbAtom::MANAGER))
+        if (m_systemTrayTracker && clientMessage->type == atom(QXcbAtom::AtomMANAGER))
             m_systemTrayTracker->notifyManagerClientMessageEvent(clientMessage);
         HANDLE_PLATFORM_WINDOW_EVENT(xcb_client_message_event_t, window, handleClientMessageEvent);
     }
     case XCB_ENTER_NOTIFY:
-#if QT_CONFIG(xcb_xinput)
-        if (hasXInput2() && !xi2MouseEventsDisabled())
+        if (hasXInput2()) {
+            // Prefer XI2 enter (XCB_INPUT_ENTER) events over core events.
             break;
-#endif
+        }
+        setTime(reinterpret_cast<xcb_enter_notify_event_t *>(event)->time);
         HANDLE_PLATFORM_WINDOW_EVENT(xcb_enter_notify_event_t, event, handleEnterNotifyEvent);
     case XCB_LEAVE_NOTIFY:
-#if QT_CONFIG(xcb_xinput)
-        if (hasXInput2() && !xi2MouseEventsDisabled())
+    {
+        if (hasXInput2()) {
+            // Prefer XI2 leave (XCB_INPUT_LEAVE) events over core events.
             break;
-#endif
-        m_keyboard->updateXKBStateFromCore(reinterpret_cast<xcb_leave_notify_event_t *>(event)->state);
+        }
+        auto ev = reinterpret_cast<xcb_leave_notify_event_t *>(event);
+        setTime(ev->time);
+        m_keyboard->updateXKBStateFromCore(ev->state);
         HANDLE_PLATFORM_WINDOW_EVENT(xcb_leave_notify_event_t, event, handleLeaveNotifyEvent);
+    }
     case XCB_FOCUS_IN:
         HANDLE_PLATFORM_WINDOW_EVENT(xcb_focus_in_event_t, event, handleFocusInEvent);
     case XCB_FOCUS_OUT:
@@ -615,13 +636,18 @@ void QXcbConnection::handleXcbEvent(xcb_generic_event_t *event)
     case XCB_KEY_PRESS:
     {
         auto keyPress = reinterpret_cast<xcb_key_press_event_t *>(event);
+        setTime(keyPress->time);
         m_keyboard->updateXKBStateFromCore(keyPress->state);
         setTime(keyPress->time);
         HANDLE_KEYBOARD_EVENT(xcb_key_press_event_t, handleKeyPressEvent);
     }
     case XCB_KEY_RELEASE:
-        m_keyboard->updateXKBStateFromCore(reinterpret_cast<xcb_key_release_event_t *>(event)->state);
+    {
+        auto keyRelease = reinterpret_cast<xcb_key_release_event_t *>(event);
+        setTime(keyRelease->time);
+        m_keyboard->updateXKBStateFromCore(keyRelease->state);
         HANDLE_KEYBOARD_EVENT(xcb_key_release_event_t, handleKeyReleaseEvent);
+    }
     case XCB_MAPPING_NOTIFY:
         m_keyboard->updateKeymap(reinterpret_cast<xcb_mapping_notify_event_t *>(event));
         break;
@@ -629,9 +655,10 @@ void QXcbConnection::handleXcbEvent(xcb_generic_event_t *event)
     {
 #if QT_CONFIG(draganddrop) || QT_CONFIG(clipboard)
         auto selectionRequest = reinterpret_cast<xcb_selection_request_event_t *>(event);
+        setTime(selectionRequest->time);
 #endif
 #if QT_CONFIG(draganddrop)
-        if (selectionRequest->selection == atom(QXcbAtom::XdndSelection))
+        if (selectionRequest->selection == atom(QXcbAtom::AtomXdndSelection))
             m_drag->handleSelectionRequest(selectionRequest);
         else
 #endif
@@ -653,27 +680,28 @@ void QXcbConnection::handleXcbEvent(xcb_generic_event_t *event)
         break;
     case XCB_PROPERTY_NOTIFY:
     {
+        auto propertyNotify = reinterpret_cast<xcb_property_notify_event_t *>(event);
+        setTime(propertyNotify->time);
 #ifndef QT_NO_CLIPBOARD
         if (m_clipboard->handlePropertyNotify(event))
             break;
 #endif
-        auto propertyNotify = reinterpret_cast<xcb_property_notify_event_t *>(event);
-        if (propertyNotify->atom == atom(QXcbAtom::_NET_WORKAREA)) {
+        if (propertyNotify->atom == atom(QXcbAtom::Atom_NET_WORKAREA)) {
             QXcbVirtualDesktop *virtualDesktop = virtualDesktopForRootWindow(propertyNotify->window);
             if (virtualDesktop)
                 virtualDesktop->updateWorkArea();
+        } else if (propertyNotify->atom == atom(QXcbAtom::Atom_NET_SUPPORTED)) {
+            m_wmSupport->updateNetWMAtoms();
         } else {
             HANDLE_PLATFORM_WINDOW_EVENT(xcb_property_notify_event_t, window, handlePropertyNotifyEvent);
         }
         break;
     }
-#if QT_CONFIG(xcb_xinput)
     case XCB_GE_GENERIC:
         // Here the windowEventListener is invoked from xi2HandleEvent()
         if (hasXInput2() && isXIEvent(event))
             xi2HandleEvent(reinterpret_cast<xcb_ge_event_t *>(event));
         break;
-#endif
     default:
         handled = false; // event type not recognized
         break;
@@ -689,21 +717,23 @@ void QXcbConnection::handleXcbEvent(xcb_generic_event_t *event)
 #ifndef QT_NO_CLIPBOARD
         m_clipboard->handleXFixesSelectionRequest(notify_event);
 #endif
-        for (QXcbVirtualDesktop *virtualDesktop : qAsConst(m_virtualDesktops))
+        for (QXcbVirtualDesktop *virtualDesktop : std::as_const(m_virtualDesktops))
             virtualDesktop->handleXFixesSelectionNotify(notify_event);
     } else if (isXRandrType(response_type, XCB_RANDR_NOTIFY)) {
-        updateScreens(reinterpret_cast<xcb_randr_notify_event_t *>(event));
+        if (!isAtLeastXRandR15())
+            updateScreens(reinterpret_cast<xcb_randr_notify_event_t *>(event));
     } else if (isXRandrType(response_type, XCB_RANDR_SCREEN_CHANGE_NOTIFY)) {
-        auto change_event = reinterpret_cast<xcb_randr_screen_change_notify_event_t *>(event);
-        if (auto virtualDesktop = virtualDesktopForRootWindow(change_event->root))
-            virtualDesktop->handleScreenChange(change_event);
-#if QT_CONFIG(xkb)
+        if (!isAtLeastXRandR15()) {
+            auto change_event = reinterpret_cast<xcb_randr_screen_change_notify_event_t *>(event);
+            if (auto virtualDesktop = virtualDesktopForRootWindow(change_event->root))
+                virtualDesktop->handleScreenChange(change_event);
+        }
     } else if (isXkbType(response_type)) {
         auto xkb_event = reinterpret_cast<_xkb_event *>(event);
         if (xkb_event->any.deviceID == m_keyboard->coreDeviceId()) {
             switch (xkb_event->any.xkbType) {
                 // XkbNewKkdNotify and XkbMapNotify together capture all sorts of keymap
-                // updates (e.g. xmodmap, xkbcomp, setxkbmap), with minimal redundent recompilations.
+                // updates (e.g. xmodmap, xkbcomp, setxkbmap), with minimal redundant recompilations.
                 case XCB_XKB_STATE_NOTIFY:
                     m_keyboard->updateXKBState(&xkb_event->state_notify);
                     break;
@@ -720,7 +750,6 @@ void QXcbConnection::handleXcbEvent(xcb_generic_event_t *event)
                     break;
             }
         }
-#endif
     } else {
         handled = false; // event type still not recognized
     }
@@ -732,11 +761,6 @@ void QXcbConnection::handleXcbEvent(xcb_generic_event_t *event)
         m_glIntegration->handleXcbEvent(event, response_type);
 }
 
-void QXcbConnection::addPeekFunc(PeekFunc f)
-{
-    m_peekFuncs.append(f);
-}
-
 void QXcbConnection::setFocusWindow(QWindow *w)
 {
     m_focusWindow = w ? static_cast<QXcbWindow *>(w->handle()) : nullptr;
@@ -751,6 +775,28 @@ void QXcbConnection::setMousePressWindow(QXcbWindow *w)
     m_mousePressWindow = w;
 }
 
+QByteArray QXcbConnection::startupId() const
+{
+    return m_startupId;
+}
+void QXcbConnection::setStartupId(const QByteArray &nextId)
+{
+    m_startupId = nextId;
+    if (m_clientLeader) {
+        if (!nextId.isEmpty())
+            xcb_change_property(xcb_connection(),
+                                XCB_PROP_MODE_REPLACE,
+                                clientLeader(),
+                                atom(QXcbAtom::Atom_NET_STARTUP_ID),
+                                atom(QXcbAtom::AtomUTF8_STRING),
+                                8,
+                                nextId.size(),
+                                nextId.constData());
+        else
+            xcb_delete_property(xcb_connection(), clientLeader(), atom(QXcbAtom::Atom_NET_STARTUP_ID));
+    }
+}
+
 void QXcbConnection::grabServer()
 {
     if (m_canGrabServer)
@@ -763,19 +809,31 @@ void QXcbConnection::ungrabServer()
         xcb_ungrab_server(xcb_connection());
 }
 
+QString QXcbConnection::windowManagerName() const
+{
+    QXcbVirtualDesktop *pvd = primaryVirtualDesktop();
+    if (pvd)
+        return pvd->windowManagerName().toLower();
+
+    return QString();
+}
+
 xcb_timestamp_t QXcbConnection::getTimestamp()
 {
     // send a dummy event to myself to get the timestamp from X server.
     xcb_window_t window = rootWindow();
-    xcb_atom_t dummyAtom = atom(QXcbAtom::CLIP_TEMPORARY);
-    xcb_change_property(xcb_connection(), XCB_PROP_MODE_APPEND, window, dummyAtom,
+    xcb_atom_t dummyAtom = atom(QXcbAtom::Atom_QT_GET_TIMESTAMP);
+    xcb_change_property(xcb_connection(), XCB_PROP_MODE_REPLACE, window, dummyAtom,
                         XCB_ATOM_INTEGER, 32, 0, nullptr);
 
     connection()->flush();
 
     xcb_generic_event_t *event = nullptr;
 
-    while (!event) {
+    // When disconnection is caused by X server, event will never be able to hold
+    // a valid pointer. isConnected(), which calls xcb_connection_has_error(),
+    // can handle this type of disconnection and properly quits the loop.
+    while (isConnected() && !event) {
         connection()->sync();
         event = eventQueue()->peek([window, dummyAtom](xcb_generic_event_t *event, int type) {
             if (type != XCB_PROPERTY_NOTIFY)
@@ -785,21 +843,33 @@ xcb_timestamp_t QXcbConnection::getTimestamp()
         });
     }
 
+    if (!event) {
+        // https://www.x.org/releases/X11R7.7/doc/xproto/x11protocol.html#glossary
+        // > One timestamp value (named CurrentTime) is never generated by the
+        // > server. This value is reserved for use in requests to represent the
+        // > current server time.
+        return XCB_CURRENT_TIME;
+    }
+
     xcb_property_notify_event_t *pn = reinterpret_cast<xcb_property_notify_event_t *>(event);
     xcb_timestamp_t timestamp = pn->time;
     free(event);
 
-    xcb_delete_property(xcb_connection(), window, dummyAtom);
-
     return timestamp;
 }
 
-xcb_window_t QXcbConnection::getSelectionOwner(xcb_atom_t atom) const
+xcb_window_t QXcbConnection::selectionOwner(xcb_atom_t atom) const
 {
-    return Q_XCB_REPLY(xcb_get_selection_owner, xcb_connection(), atom)->owner;
+    auto reply = Q_XCB_REPLY(xcb_get_selection_owner, xcb_connection(), atom);
+    if (!reply) {
+        qCDebug(lcQpaXcb) << "failed to query selection owner";
+        return XCB_NONE;
+    }
+
+    return reply->owner;
 }
 
-xcb_window_t QXcbConnection::getQtSelectionOwner()
+xcb_window_t QXcbConnection::qtSelectionOwner()
 {
     if (!m_qtSelectionOwner) {
         xcb_screen_t *xcbScreen = primaryVirtualDesktop()->screen();
@@ -815,10 +885,10 @@ xcb_window_t QXcbConnection::getQtSelectionOwner()
                           XCB_WINDOW_CLASS_INPUT_OUTPUT,      // window class
                           xcbScreen->root_visual,             // visual
                           0,                                  // value mask
-                          0);                                 // value list
+                          nullptr);                           // value list
 
         QXcbWindow::setWindowTitle(connection(), m_qtSelectionOwner,
-                                   QLatin1String("Qt Selection Owner for ") + QCoreApplication::applicationName());
+                                   "Qt Selection Owner for "_L1 + QCoreApplication::applicationName());
     }
     return m_qtSelectionOwner;
 }
@@ -842,16 +912,16 @@ xcb_window_t QXcbConnection::clientLeader()
                           0,
                           XCB_WINDOW_CLASS_INPUT_OUTPUT,
                           screen->screen()->root_visual,
-                          0, 0);
+                          0, nullptr);
 
 
         QXcbWindow::setWindowTitle(connection(), m_clientLeader,
-                                   QStringLiteral("Qt Client Leader Window"));
+                                   QGuiApplication::applicationDisplayName());
 
         xcb_change_property(xcb_connection(),
                             XCB_PROP_MODE_REPLACE,
                             m_clientLeader,
-                            atom(QXcbAtom::WM_CLIENT_LEADER),
+                            atom(QXcbAtom::AtomWM_CLIENT_LEADER),
                             XCB_ATOM_WINDOW,
                             32,
                             1,
@@ -864,13 +934,15 @@ xcb_window_t QXcbConnection::clientLeader()
             xcb_change_property(xcb_connection(),
                                 XCB_PROP_MODE_REPLACE,
                                 m_clientLeader,
-                                atom(QXcbAtom::SM_CLIENT_ID),
+                                atom(QXcbAtom::AtomSM_CLIENT_ID),
                                 XCB_ATOM_STRING,
                                 8,
-                                session.length(),
+                                session.size(),
                                 session.constData());
         }
 #endif
+
+        setStartupId(startupId());
     }
     return m_clientLeader;
 }
@@ -902,7 +974,6 @@ bool QXcbConnection::compressEvent(xcb_generic_event_t *event) const
         });
     }
 
-#if QT_CONFIG(xcb_xinput)
     // compress XI_* events
     if (responseType == XCB_GE_GENERIC) {
         if (!hasXInput2())
@@ -938,7 +1009,6 @@ bool QXcbConnection::compressEvent(xcb_generic_event_t *event) const
 
         return false;
     }
-#endif
 
     if (responseType == XCB_CONFIGURE_NOTIFY) {
         // compress multiple configure notify events for the same window
@@ -968,7 +1038,6 @@ bool QXcbConnection::isUserInputEvent(xcb_generic_event_t *event) const
     if (isInputEvent)
         return true;
 
-#if QT_CONFIG(xcb_xinput)
     if (connection()->hasXInput2()) {
         isInputEvent = isXIType(event, XCB_INPUT_BUTTON_PRESS) ||
                        isXIType(event, XCB_INPUT_BUTTON_RELEASE) ||
@@ -983,12 +1052,11 @@ bool QXcbConnection::isUserInputEvent(xcb_generic_event_t *event) const
     }
     if (isInputEvent)
         return true;
-#endif
 
     if (eventType == XCB_CLIENT_MESSAGE) {
         auto clientMessage = reinterpret_cast<const xcb_client_message_event_t *>(event);
-        if (clientMessage->format == 32 && clientMessage->type == atom(QXcbAtom::WM_PROTOCOLS))
-            if (clientMessage->data.data32[0] == atom(QXcbAtom::WM_DELETE_WINDOW))
+        if (clientMessage->format == 32 && clientMessage->type == atom(QXcbAtom::AtomWM_PROTOCOLS))
+            if (clientMessage->data.data32[0] == atom(QXcbAtom::AtomWM_DELETE_WINDOW))
                 isInputEvent = true;
     }
 
@@ -1016,15 +1084,6 @@ void QXcbConnection::processXcbEvents(QEventLoop::ProcessEventsFlags flags)
         if (compressEvent(event))
             continue;
 
-        auto isWaitingFor = [=](PeekFunc peekFunc) {
-            // These callbacks return true if the event is what they were
-            // waiting for, remove them from the list in that case.
-            return peekFunc(this, event);
-        };
-        m_peekFuncs.erase(std::remove_if(m_peekFuncs.begin(), m_peekFuncs.end(),
-                                         isWaitingFor),
-                          m_peekFuncs.end());
-
         handleXcbEvent(event);
 
         // The lock-based solution used to free the lock inside this loop,
@@ -1033,11 +1092,9 @@ void QXcbConnection::processXcbEvents(QEventLoop::ProcessEventsFlags flags)
         m_eventQueue->flushBufferedEvents();
     }
 
-    // Indicate with a null event that the event the callbacks are waiting for
-    // is not in the queue currently.
-    for (PeekFunc f : qAsConst(m_peekFuncs))
-        f(this, nullptr);
-    m_peekFuncs.clear();
+#if QT_CONFIG(xcb_xlib)
+    qt_XFlush(static_cast<Display *>(xlib_display()));
+#endif
 
     xcb_flush(xcb_connection());
 }
@@ -1062,7 +1119,7 @@ void QXcbConnection::sync()
 {
     // from xcb_aux_sync
     xcb_get_input_focus_cookie_t cookie = xcb_get_input_focus(xcb_connection());
-    free(xcb_get_input_focus_reply(xcb_connection(), cookie, 0));
+    free(xcb_get_input_focus_reply(xcb_connection(), cookie, nullptr));
 }
 
 QXcbSystemTrayTracker *QXcbConnection::systemTrayTracker() const
@@ -1080,14 +1137,14 @@ QXcbSystemTrayTracker *QXcbConnection::systemTrayTracker() const
 Qt::MouseButtons QXcbConnection::queryMouseButtons() const
 {
     int stateMask = 0;
-    QXcbCursor::queryPointer(connection(), 0, 0, &stateMask);
+    QXcbCursor::queryPointer(connection(), nullptr, nullptr, &stateMask);
     return translateMouseButtons(stateMask);
 }
 
 Qt::KeyboardModifiers QXcbConnection::queryKeyboardModifiers() const
 {
     int stateMask = 0;
-    QXcbCursor::queryPointer(connection(), 0, 0, &stateMask);
+    QXcbCursor::queryPointer(connection(), nullptr, nullptr, &stateMask);
     return keyboard()->translateModifiers(stateMask);
 }
 
@@ -1101,7 +1158,7 @@ QXcbGlIntegration *QXcbConnection::glIntegration() const
     QString glIntegrationName = QString::fromLocal8Bit(qgetenv("QT_XCB_GL_INTEGRATION"));
     if (!glIntegrationName.isEmpty()) {
         qCDebug(lcQpaGl) << "QT_XCB_GL_INTEGRATION is set to" << glIntegrationName;
-        if (glIntegrationName != QLatin1String("none")) {
+        if (glIntegrationName != "none"_L1) {
             glIntegrationNames.removeAll(glIntegrationName);
             glIntegrationNames.prepend(glIntegrationName);
         } else {
@@ -1145,7 +1202,7 @@ void QXcbSyncWindowRequest::invalidate()
 {
     if (m_window) {
         m_window->clearSyncWindowRequest();
-        m_window = 0;
+        m_window = nullptr;
     }
 }
 
@@ -1165,8 +1222,10 @@ void QXcbConnectionGrabber::release()
 {
     if (m_connection) {
         m_connection->ungrabServer();
-        m_connection = 0;
+        m_connection = nullptr;
     }
 }
 
 QT_END_NAMESPACE
+
+#include "moc_qxcbconnection.cpp"
