@@ -342,10 +342,9 @@ bool QSqlTableModel::select()
 
     d->clearCache();
 
-    QSqlQuery qu(query, d->db);
-    setQuery(qu);
+    this->QSqlQueryModel::setQuery(query, d->db);
 
-    if (!qu.isActive() || lastError().isValid()) {
+    if (!d->query.isActive() || lastError().isValid()) {
         // something went wrong - revert to non-select state
         d->initRecordAndPrimaryIndex();
         endResetModel();
@@ -581,18 +580,6 @@ bool QSqlTableModel::clearItemData(const QModelIndex &index)
     return setData(index, QVariant(), Qt::EditRole);
 }
 
-/*!
-    This function simply calls QSqlQueryModel::setQuery(\a query).
-    You should normally not call it on a QSqlTableModel. Instead, use
-    setTable(), setSort(), setFilter(), etc., to set up the query.
-
-    \sa selectStatement()
-*/
-void QSqlTableModel::setQuery(const QSqlQuery &query)
-{
-    QT_IGNORE_DEPRECATIONS(QSqlQueryModel::setQuery(query);)
-}
-
 /*!
     Updates the given \a row in the currently active database table
     with the specified \a values. Returns \c true if successful; otherwise
